@@ -471,7 +471,7 @@ void pushGenericCommand(client *c, int where, int xx) {
         }
 
         lobj = createListListpackObject();
-        dbAdd(c->db, c->argv[1], lobj);
+        dbAdd(c->db, c->argv[1], &lobj);
     }
 
     listTypeTryConversionAppend(lobj, c->argv, 2, c->argc - 1, NULL, NULL);
@@ -907,9 +907,8 @@ void ltrimCommand(client *c) {
     } else {
         listTypeTryConversion(o, LIST_CONV_SHRINKING, NULL, NULL);
     }
-    long long total_trim = ltrim + rtrim;
-    if (total_trim) signalModifiedKey(c, c->db, c->argv[1]);
-    server.dirty += total_trim;
+    signalModifiedKey(c, c->db, c->argv[1]);
+    server.dirty += (ltrim + rtrim);
     addReply(c, shared.ok);
 }
 
@@ -1069,7 +1068,7 @@ void lmoveHandlePush(client *c, robj *dstkey, robj *dstobj, robj *value, int whe
     /* Create the list if the key does not exist */
     if (!dstobj) {
         dstobj = createListListpackObject();
-        dbAdd(c->db, dstkey, dstobj);
+        dbAdd(c->db, dstkey, &dstobj);
     }
     listTypeTryConversionAppend(dstobj, &value, 0, 0, NULL, NULL);
     listTypePush(dstobj, value, where);
